local _, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:GetModule("Plus")

local _G = getfenv(0)
local select, pairs, type = select, pairs, type

local r, g, b = DB.r, DB.g, DB.b

local function expandOnEnter(self)
	if self:IsEnabled() then
		self.bg:SetBackdropColor(r, g, b, .25)
	end
end

local function expandOnLeave(self)
	self.bg:SetBackdropColor(0, 0, 0, .25)
end

local function SetupTexture(self, texture)
	if self.settingTexture then return end
	self.settingTexture = true
	self:SetNormalTexture("")

	if texture and texture ~= "" then
		if type(texture) == "number" then
			if texture == 130838 then
				self.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture == 130821 then
				self.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		else
			if texture:find("Plus") then
				self.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture:find("Minus") then
				self.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		end
		self.bg:Show()
	else
		self.bg:Hide()
	end
	self.settingTexture = nil
end

function P:ReskinExpandOrCollapse()
	self:SetHighlightTexture("")
	self:SetPushedTexture("")

	local bg = B.CreateBDFrame(self, .25)
	bg:ClearAllPoints()
	bg:SetSize(13, 13)
	bg:SetPoint("TOPLEFT", self:GetNormalTexture())
	B.CreateGradient(bg)
	self.bg = bg

	self.expTex = bg:CreateTexture(nil, "OVERLAY")
	self.expTex:SetSize(7, 7)
	self.expTex:SetPoint("CENTER")
	self.expTex:SetTexture("Interface\\Buttons\\UI-PlusMinus-Buttons")

	self:HookScript("OnEnter", expandOnEnter)
	self:HookScript("OnLeave", expandOnLeave)
	hooksecurefunc(self, "SetNormalTexture", SetupTexture)
end

local function TexSetupTexture(self, texture)
	local frame = self:GetParent()
	if self.settingTexture then return end
	self.settingTexture = true
	self:SetTexture("")

	if texture and texture ~= "" then
		if type(texture) == "number" then
			if texture == 130838 then
				frame.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture == 130821 then
				frame.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		else
			if texture:find("plus") then
				frame.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture:find("minus") then
				frame.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		end
		frame.bg:Show()
	else
		frame.bg:Hide()
	end
	self.settingTexture = nil
end

function P:ReskinExpandOrCollapseTex()
	self:SetTexture("")
	local frame = self:GetParent()

	local bg = B.CreateBDFrame(frame, .25)
	bg:ClearAllPoints()
	bg:SetSize(13, 13)
	bg:SetPoint("TOPLEFT", frame)
	B.CreateGradient(bg)
	frame.bg = bg

	frame.expTex = bg:CreateTexture(nil, "OVERLAY")
	frame.expTex:SetSize(7, 7)
	frame.expTex:SetPoint("CENTER")
	frame.expTex:SetTexture("Interface\\Buttons\\UI-PlusMinus-Buttons")

	frame:HookScript("OnEnter", expandOnEnter)
	frame:HookScript("OnLeave", expandOnLeave)
	hooksecurefunc(self, "SetTexture", TexSetupTexture)
	hooksecurefunc(self, "Show", function()
		self:GetParent().bg:Show()
		self:GetParent().expTex:Show()
	end)
	hooksecurefunc(self, "Hide", function()
		self:GetParent().bg:Hide()
		self:GetParent().expTex:Hide()
	end)
end

function P.ReskinFont(font, size)
	local oldSize = select(2, font:GetFont())
	size = size or oldSize
	local fontSize = size*NDuiDB["Skins"]["FontScale"]
	font:SetFont(DB.Font[1], fontSize, DB.Font[3])
	font:SetShadowColor(0, 0, 0, 0)
end

function P:AnchorTooltip()
    if self:GetRight() >= (GetScreenWidth() / 2) then
        GameTooltip:SetOwner(self, 'ANCHOR_LEFT')
    else
        GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
    end
end

function P.NameGenerator(name)
    local index = 0
    return function()
        index = index + 1
        return name .. index
    end
end

P.LEFT_MOUSE_BUTTON = [[|TInterface\TutorialFrame\UI-Tutorial-Frame:12:12:0:0:512:512:10:65:228:283|t]]
P.RIGHT_MOUSE_BUTTON = [[|TInterface\TutorialFrame\UI-Tutorial-Frame:12:12:0:0:512:512:10:65:330:385|t]]

function P.LeftButtonTip(text)
    return P.LEFT_MOUSE_BUTTON .. text, 1, 1, 1
end

function P.RightButtonTip(text)
    return P.RIGHT_MOUSE_BUTTON .. text, 1, 1, 1
end

do
	local PIXEL_BORDERS = {"TOPLEFT", "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT", "TOP", "BOTTOM", "LEFT", "RIGHT"}

	function P:SetBackdrop(frame, a)
		local borders = frame.pixelBorders
		if not borders then return end
		local size = C.mult
		borders.CENTER:SetPoint("TOPLEFT", frame)
		borders.CENTER:SetPoint("BOTTOMRIGHT", frame)

		borders.TOPLEFT:SetSize(size, size)
		borders.TOPRIGHT:SetSize(size, size)
		borders.BOTTOMLEFT:SetSize(size, size)
		borders.BOTTOMRIGHT:SetSize(size, size)

		borders.TOP:SetHeight(size)
		borders.BOTTOM:SetHeight(size)
		borders.LEFT:SetWidth(size)
		borders.RIGHT:SetWidth(size)
		P:SetBackdropColor(frame, 0, 0, 0, a)
		P:SetBackdropBorderColor(frame, 0, 0, 0)
	end
	function P:SetBackdropColor(frame, r, g, b, a)
		if frame.pixelBorders then
			frame.pixelBorders.CENTER:SetVertexColor(r, g, b, a)
		end
	end
	function P:SetBackdropBorderColor(frame, r, g, b, a)
		if frame.pixelBorders then
			for _, v in pairs(PIXEL_BORDERS) do
				frame.pixelBorders[v]:SetVertexColor(r or 0, g or 0, b or 0, a)
			end
		end
	end
	function P:SetBackdropColor_Hook(r, g, b, a)
		P:SetBackdropColor(self, r, g, b, a)
	end
	function P:SetBackdropBorderColor_Hook(r, g, b, a)
		P:SetBackdropBorderColor(self, r, g, b, a)
	end
	function P:PixelBorders(frame)
		if frame and not frame.pixelBorders then
			local borders = {}
			for _, v in pairs(PIXEL_BORDERS) do
				borders[v] = frame:CreateTexture(nil, "BORDER", nil, 1)
				borders[v]:SetTexture(DB.bdTex)
			end
			borders.CENTER = frame:CreateTexture(nil, "BACKGROUND", nil, -1)
			borders.CENTER:SetTexture(DB.bdTex)

			borders.TOPLEFT:Point("BOTTOMRIGHT", borders.CENTER, "TOPLEFT", C.mult, -C.mult)
			borders.TOPRIGHT:Point("BOTTOMLEFT", borders.CENTER, "TOPRIGHT", -C.mult, -C.mult)
			borders.BOTTOMLEFT:Point("TOPRIGHT", borders.CENTER, "BOTTOMLEFT", C.mult, C.mult)
			borders.BOTTOMRIGHT:Point("TOPLEFT", borders.CENTER, "BOTTOMRIGHT", -C.mult, C.mult)

			borders.TOP:Point("TOPLEFT", borders.TOPLEFT, "TOPRIGHT", 0, 0)
			borders.TOP:Point("TOPRIGHT", borders.TOPRIGHT, "TOPLEFT", 0, 0)

			borders.BOTTOM:Point("BOTTOMLEFT", borders.BOTTOMLEFT, "BOTTOMRIGHT", 0, 0)
			borders.BOTTOM:Point("BOTTOMRIGHT", borders.BOTTOMRIGHT, "BOTTOMLEFT", 0, 0)

			borders.LEFT:Point("TOPLEFT", borders.TOPLEFT, "BOTTOMLEFT", 0, 0)
			borders.LEFT:Point("BOTTOMLEFT", borders.BOTTOMLEFT, "TOPLEFT", 0, 0)

			borders.RIGHT:Point("TOPRIGHT", borders.TOPRIGHT, "BOTTOMRIGHT", 0, 0)
			borders.RIGHT:Point("BOTTOMRIGHT", borders.BOTTOMRIGHT, "TOPRIGHT", 0, 0)

			hooksecurefunc(frame, "SetBackdropColor", P.SetBackdropColor_Hook)
			hooksecurefunc(frame, "SetBackdropBorderColor", P.SetBackdropBorderColor_Hook)
			frame.pixelBorders = borders
		end
	end

	function P:CreateBD(a)
		self:SetBackdrop(nil)
		P:PixelBorders(self)
		P:SetBackdrop(self, a or NDuiDB["Skins"]["SkinAlpha"])
	end
end