local addonName, ns = ...
local B, C, L, DB = unpack(ns)
local P = B:GetModule("Plus")
local BagsModule = B:GetModule("Bags")
----------------------------
-- Credit: tdBag2
----------------------------
local BAGS = {0, 1, 2, 3, 4, -2}
local BANKS = {-1, 5, 6, 7, 8, 9, 10}
local PlayerDB
local Owner, Owners
local bagFrame, bankFrame, SelectorDropMenu
local atBank
local ItemButtons = {}

local function ParseItem(link, count, timeout)
	if link then
		if link:find('0:0:0:0:0:%d+:%d+:%d+:0:0') then
			link = link:match('|H%l+:(%d+)')
		else
			link = link:match('|H%l+:([%d:]+)')
		end

		local count = count and count > 1 and count or nil
		if count or timeout then
			link = link .. ';' .. (count or '')
		end
		if timeout then
			link = link .. ';' .. timeout
		end

		return link
	end
end

local function SaveBag(bag)
	local size = GetContainerNumSlots(bag)
	local items
	if size > 0 then
		items = {}
		items.size = size

		for slot = 1, size do
			local _, count, _, _, _, _, link = GetContainerItemInfo(bag, slot)
			items[slot] = ParseItem(link, count)
		end
	end

	PlayerDB[bag] = items
end

local function UpdateBag(_, bag)
	if bag <= NUM_BAG_SLOTS then
		SaveBag(bag)
	end
end

local function SaveMoney()
	PlayerDB.money = GetMoney()
end

local function UpdateAllDB()
	for _, bag in ipairs(BAGS) do
		SaveBag(bag)
	end

	SaveMoney()
end

local function Bank_Opened()
	atBank = true
end

local function Bank_Closed()
	if atBank then
		for _, bag in ipairs(BANKS) do
			SaveBag(bag)
		end
		atBank = nil
	end
end

local function getData(itemData)
	local data = {}
	local link, count, timeout = strsplit(';', itemData)
	data.link = 'item:' .. link
	data.count = tonumber(count)
	data.id = tonumber(link:match('^(%d+)'))
	data.icon = GetItemIcon(data.id)
	data.timeout = tonumber(timeout)

	local name, link, quality = GetItemInfo(data.link)
	if name then
		data.link = link
		data.quality = quality
	end

	return data
end

local function getOwnerDB(owner)
	local data = NDuiPlusBAGDB[DB.MyRealm][owner]
	data.name = owner

	return data
end

local function ItemButton_OnEnter(self)
	if self.Link then
		P.AnchorTooltip(self)
		GameTooltip:SetHyperlink(self.Link)
		GameTooltip:Show()
	end
end

local function ItemButton_OnClick(self)
	if self.Link then
		HandleModifiedItemClick(self.Link)
	end
end

local function ItemButton_LoadItemInfo(self, itemData)
	local data = getData(itemData)
	self.Icon:SetTexture(data.icon)
	self.Link = data.link

	if data.count and  data.count > 1 then
		self.Count:SetText(data.count)
		self.Count:Show()
	else
		self.Count:Hide()
	end

	if data.quality and data.quality > -1 then
		local color = DB.QualityColors[data.quality]
		self:SetBackdropBorderColor(color.r, color.g, color.b)
	else
		self:SetBackdropBorderColor(0, 0, 0)
	end
end

local function ItemButton_Free(self)
	self:Hide()
	self.Icon:SetTexture(nil)
	self.Link = nil
	self.Count:SetText("")
	self.Count:Hide()
	self:SetBackdropBorderColor(0, 0, 0)
end

local function CreateItemButton(bag, slot)
	local name = format(addonName.."_Bag%dSlot%d", bag, slot)
	local button = CreateFrame("Button", name, nil, "ItemButtonTemplate")
	local iconSize = NDuiDB["Bags"]["IconSize"]

	button:SetNormalTexture(nil)
	button:SetPushedTexture(nil)
	button:SetHighlightTexture(DB.bdTex)
	button:GetHighlightTexture():SetVertexColor(1, 1, 1, .25)
	button:GetHighlightTexture():SetInside()
	button:SetSize(iconSize, iconSize)
	button:Hide()

	button.Icon = _G[name.."IconTexture"]
	button.Icon:SetInside()
	button.Icon:SetTexCoord(unpack(DB.TexCoord))
	button.Count = _G[name.."Count"]
	button.Count:SetPoint("BOTTOMRIGHT", -1, 2)
	button.Count:SetFont(unpack(DB.Font))
	button.bagID = bag
	button.slotID = slot

	B.CreateBD(button, .3)
	button:SetBackdropColor(.3, .3, .3, .3)

	button:SetScript("OnEnter", ItemButton_OnEnter)
	button:SetScript("OnLeave", B.HideTooltip)
	button:SetScript("OnClick", ItemButton_OnClick)

	button.LoadItemInfo = ItemButton_LoadItemInfo
	button.Free = ItemButton_Free

	ItemButtons[bag][slot] = button

	return button
end

local function GetItemButton(bag, slot)
	return ItemButtons[bag][slot] or CreateItemButton(bag, slot)
end

local function getMoneyString(money)
	money = money or 0

	local str
	local g,s,c = floor(money/1e4), floor(money/100) % 100, money % 100

	if(g > 0) then str = (str and str.." " or "") .. g .. "|TInterface\\MoneyFrame\\UI-GoldIcon:16:16:0:0|t" end
	if(s > 0) then str = (str and str.." " or "") .. s .. "|TInterface\\MoneyFrame\\UI-SilverIcon:16:16:0:0|t" end
	if(c >= 0) then str = (str and str.." " or "") .. c .. "|TInterface\\MoneyFrame\\UI-CopperIcon:16:16:0:0|t" end
	return str
end

local function UpdateMoney()
	local ownerDB = getOwnerDB(Owner)
	local money = ownerDB.money
	local moneyStr = getMoneyString(money)
	if money then
		bagFrame.Money:SetText(moneyStr)
	end
end

local function UpdateName()
	local ownerDB = getOwnerDB(Owner)
	local class = ownerDB.class
	local r, g, b = DB.ClassColors[class].r, DB.ClassColors[class].g, DB.ClassColors[class].b
	bagFrame.Name:SetText(Owner)
	bagFrame.Name:SetTextColor(r, g, b)
end

local function FreeAllButtons()
	for bag, buttons in pairs(ItemButtons) do
		for slot, itemButton in pairs(buttons) do
			itemButton:Free()
		end
	end
end

local function LayoutButtons(frame, bags)
	local ownerDB = getOwnerDB(Owner)
	local x, y = 0, 0
	local column = 12
	local offset = 38
	local spacing = 3
	local iconSize = NDuiDB["Bags"]["IconSize"]
	local size = iconSize + spacing
	local xOffset = 5
	local yOffset = -offset + xOffset

	for _, bag in ipairs(bags) do
		local bagData = ownerDB[bag]
		if bagData and bagData.size and bagData.size > 0 then
			for slot = 1, bagData.size do
				local itemButton = GetItemButton(bag, slot)
				local itemData = bagData[slot]
				if x == column then
					y = y + 1
					x = 0
				end
				itemButton:SetParent(frame)
				itemButton:ClearAllPoints()
				itemButton:SetPoint('TOPLEFT', frame, 'TOPLEFT', x * size + xOffset , -y * size + yOffset)
				itemButton:Show()

				if itemData then
					itemButton:LoadItemInfo(itemData)
					--LoadItemInfo(itemButton, itemData)
				end

				x = x + 1
			end
		end
	end

	local width = column * (iconSize + spacing) - spacing
	local height = (y + 1) * (iconSize + spacing) - spacing
	frame:SetSize(width + xOffset*2, height + offset)
end

local function UpdateAllBags()
	FreeAllButtons()
	UpdateMoney()
	UpdateName()
	LayoutButtons(bagFrame, BAGS)
	LayoutButtons(bankFrame, BANKS)
end

local function SetupCache()
	NDuiPlusBAGDB = NDuiPlusBAGDB or {}
	NDuiPlusBAGDB[DB.MyRealm] = NDuiPlusBAGDB[DB.MyRealm] or {}
	NDuiPlusBAGDB[DB.MyRealm][DB.MyName] = NDuiPlusBAGDB[DB.MyRealm][DB.MyName] or {}
	PlayerDB = NDuiPlusBAGDB[DB.MyRealm][DB.MyName]
	PlayerDB.class = DB.MyClass

	Owner = DB.MyName
	Owners = {DB.MyName}
	for k in pairs(NDuiPlusBAGDB[DB.MyRealm]) do
		if k ~= DB.MyName then
			tinsert(Owners, k)
		end
	end

	for _, bag in ipairs(BAGS) do
		ItemButtons[bag] = {}
	end

	for _, bag in ipairs(BANKS) do
		ItemButtons[bag] = {}
	end
end

local function CreateCloseButton(parent)
	local bu = B.CreateButton(parent, 24, 24, true, "Interface\\RAIDFRAME\\ReadyCheck-NotReady")
	bu:RegisterForClicks("AnyUp")
	bu:SetScript("OnClick", function(self, btn)
		if btn == 'RightButton' then
			ToggleAllBags()
		end
		parent:Hide()
	end)
	bu.title = CLOSE
	B.AddTooltip(bu, "ANCHOR_TOP", P.RightButtonTip(L["Open Bag"]))

	return bu
end

local function CreateDropMenu()
	if not SelectorDropMenu then
		local frame = CreateFrame('Frame', "NDui_Plus_SelectorDropMenu", UIParent, 'UIDropDownMenuTemplate')
		frame.displayMode = 'MENU'
		frame.initialize = EasyMenu_Initialize

		SelectorDropMenu = frame
	end
	return SelectorDropMenu
end

local function GetOwnerColoredName(ownerDB)
	local color = RAID_CLASS_COLORS[ownerDB.class or 'PRIEST']
	return format('|cff%02x%02x%02x%s|r', color.r * 0xFF, color.g * 0xFF, color.b * 0xFF, ownerDB.name)
end

local function DeleteOwnerInfo(name)
	local realmData = NDuiPlusBAGDB[DB.MyRealm]
	if realmData then
		realmData[name] = nil
	end

	tDeleteItem(Owners, name)
end

local function CreateOwnerMenu(name)
	local isSelf = name == DB.MyName
	local isCurrent = name == Owner
	local hasArrow = not isSelf and not isCurrent
	local info = getOwnerDB(name)

	return {
		text = GetOwnerColoredName(info),
		checked = isCurrent,
		hasArrow = hasArrow,
		menuList = hasArrow and {
			{
				notCheckable = true,
				text = DELETE,
				func = function()
					DeleteOwnerInfo(name)
					CloseDropDownMenus(1)
				end,
			},
		},
		func = function()
			Owner = name
			UpdateAllBags()
		end,
	}
end

local function CreateMenu()
	local menuList = {}
	for _, owner in ipairs(Owners) do
		tinsert(menuList, CreateOwnerMenu(owner))
	end
	return menuList
end

local function IsMenuOpened()
	return SelectorDropMenu and _G.UIDROPDOWNMENU_OPEN_MENU == SelectorDropMenu and DropDownList1:IsShown()
end

local function ToggleMenu(frame)
	if IsMenuOpened() then
		CloseDropDownMenus()
		PlaySound(851)
	else
		CloseDropDownMenus()
		ToggleDropDownMenu(1, nil, SelectorDropMenu or CreateDropMenu(), frame, 0, 0, CreateMenu())
		PlaySound(850)
	end
end

local function OwnerSelector_OnClick(self, btn)
	if btn == 'RightButton' then
		Owner = DB.MyName
		UpdateAllBags()
	else
		B.HideTooltip()
		ToggleMenu(self)
	end
end

local function OwnerSelector_OnEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOP")
	GameTooltip:SetText(CHARACTER)
	GameTooltip:AddLine(P.LeftButtonTip(L.TOOLTIP_CHANGE_PLAYER))
	GameTooltip:AddLine(P.RightButtonTip(L.TOOLTIP_RETURN_TO_SELF))
	GameTooltip:Show()
end

local function CreateSelectorButton(parent)
	local bu = B.CreateButton(parent, 24, 24, true, "Interface\\CHATFRAME\\UI-ChatIcon-Battlenet")
	bu:RegisterForClicks("AnyUp")
	bu:SetScript("OnClick", OwnerSelector_OnClick)
	bu:SetScript("OnEnter", OwnerSelector_OnEnter)
	bu:SetScript("OnLeave", B.HideTooltip)

	return bu
end

local function CreateBankToggle(parent)
	local bu = B.CreateButton(parent, 24, 24, true, "Interface\\ICONS\\INV_Misc_Bag_08")
	bu:SetScript("OnClick", function()
		ToggleFrame(bankFrame)
		if bankFrame:IsShown() then
			bu:SetBackdropBorderColor(1, .8, 0)
			PlaySound(SOUNDKIT.IG_BACKPACK_OPEN)
		else
			bu:SetBackdropBorderColor(0, 0, 0)
			PlaySound(SOUNDKIT.IG_BACKPACK_CLOSE)
		end
	end)
	bu.title = BANK
	B.AddTooltip(bu, "ANCHOR_TOP")

	return bu
end

local function CreateMainFrame()
	bagFrame = CreateFrame("Frame", addonName.."_BagFrame", UIParent)
	tinsert(UISpecialFrames, addonName.."_BagFrame")
	bagFrame:SetPoint("TOPLEFT", UIParent, "LEFT", 150, 50)
	bagFrame:SetFrameStrata("HIGH")
	B.SetBD(bagFrame)
	B.CreateMF(bagFrame, nil, true)
	B.RestoreMF(bagFrame)

	bagFrame.Title = B.CreateFS(bagFrame, 14, BAGSLOT, true, "TOPLEFT", 5, -8)
	bagFrame.Money = bagFrame:CreateFontString(nil, "OVERLAY")
	bagFrame.Money:SetFont(unpack(DB.Font))
	bagFrame.Money:SetPoint("LEFT", bagFrame.Title, "RIGHT", 5, 0)
	bagFrame.Name = B.CreateFS(bagFrame, 14, DB.MyName, true, "TOP", 0, -8)
	bagFrame.Close = CreateCloseButton(bagFrame)
	bagFrame.Close:SetPoint("TOPRIGHT", -5, -3)
	bagFrame.Selector = CreateSelectorButton(bagFrame)
	bagFrame.Selector:SetPoint("RIGHT", bagFrame.Close, "LEFT", -3, 0)
	bagFrame.BankToggle = CreateBankToggle(bagFrame)
	bagFrame.BankToggle:SetPoint("RIGHT", bagFrame.Selector, "LEFT", -3, 0)
	bagFrame:Hide()

	bankFrame = CreateFrame("Frame", addonName.."_BankFrame", bagFrame)
	bankFrame:SetPoint("BOTTOMLEFT", bagFrame, "TOPLEFT", 0, 5)
	bankFrame:SetFrameStrata("HIGH")
	bankFrame:SetClampedToScreen(true)
	B.SetBD(bankFrame)
	B.CreateMF(bankFrame, bagFrame, true)
	bankFrame.Title = B.CreateFS(bankFrame, 14, BANK, true, "TOPLEFT", 5, -8)
	bankFrame:Hide()
end

local function CloseButton_OnClick(_, btn)
	if btn == 'RightButton' then
		UpdateAllBags()
		ToggleFrame(NDui_Plus_BagFrame)
	end
end

local function Backpack_OnShow(self)
	if not self.hooked then
		for i = 1, NDui_BackpackMain:GetNumChildren() do
			local child = select(i, NDui_BackpackMain:GetChildren())
			if child:GetObjectType() == "Button" and child.title and child.title == CLOSE then
				child:RegisterForClicks("AnyUp")
				child:HookScript("OnClick", CloseButton_OnClick)
				child.text = P.RightButtonTip(L["Open OfflineBag"])
				self.hooked = true
			end
		end
	end
end

function P:OfflineBag()
	if not NDuiPlusDB["OfflineBag"] then return end

	SetupCache()
	UpdateAllDB()
	CreateMainFrame()

	B:RegisterEvent("BAG_UPDATE", UpdateBag)
	B:RegisterEvent("BAG_CLOSED", UpdateBag)
	B:RegisterEvent("PLAYER_MONEY", SaveMoney)
	B:RegisterEvent("BANKFRAME_OPENED", Bank_Opened)
	B:RegisterEvent("BANKFRAME_CLOSED", Bank_Closed)

	local Backpack = BagsModule.Bags
	Backpack:HookScript("OnShow", Backpack_OnShow)
end

SlashCmdList['NDUI_PLUS_BAG'] = function(msg)
	if not bagFrame then return end

	UpdateAllBags()
	ToggleFrame(NDui_Plus_BagFrame)
end
SLASH_NDUI_PLUS_BAG1 = "/ndpb"